; Copyright 2001 by EMT Software Inc. - by Scott Hull 05/06/01
;
; updated: 07/20/01
;
; This program will draw a 2D profile of a roller sprocket.
;
; Data from Engineering Handbook  
;
(defun C:2DSPRKT (/ #DCL-FILE #DCL-ID #DCL-LIST #FILE #HELP #SIZE
 #SIZELIST #TEETH @ARC @BLOCK @INSERT @LINE @POLYARC @POLYLINE
 @SPRKT-PTS @SPROCKET-DRAW @TEETH *error*)

; start or R14 compatibility library
(if (= (substr (getvar "acadver") 1 2) "14")
 (progn
(defun vl-file-delete (%A) nil)

(defun vl-filename-mktemp (%A / #DIR)
 (cond
  ((setq #DIR (getenv "tmp")) nil)
  ((setq #DIR (getenv "temp")) nil)
  (T (setq #DIR "")))
 (strcat #DIR "\\" %A))

(defun vl-position (%A %B)
 (if (member %A %B) (- (length %B) (length (member %A %B)))))
))
; end of R14 compatibility library

 (defun *error* (%A)
  (if (= (type #FILE) 'FILE) (close #FILE))
  (cond
   ((= %A "Function cancelled") nil)
   (t (princ (strcat "\nerror: " %A "\007\n"))))
  (princ))

 (if (not (and (equal (getvar "ucsorg") (list 0.0 0.0 0.0))
               (equal (getvar "ucsxdir") (list 1.0 0.0 0.0))
               (equal (getvar "ucsydir") (list 0.0 1.0 0.0))))
  (progn (alert "You must be in the World UCS to use this program")
         (quit)))

 (setq #DCL-LIST (list
"sprocket2d : dialog {"
"  label = \"2D Sprocket\";"
"    : row {"
"      : popup_list {"
"        height = 3;    "
"        label = \"&Size\";"
"        key = \"size\";"
"        width = 16;"
"      }"
"      spacer;"
"      : row {"
"        : edit_box {"
"          edit_width = 3;"
"          key = \"teeth\";"
"          label = \"&Teeth:\";"
"        }"
"        : button {"
"          key = \"teeth_minus\";"
"          label = \"-\";"
"        }"
"        : button {"
"          key = \"teeth_plus\";"
"          label = \"+\";"
"        }"
"      }"
"    }"
"  spacer;"
"  ok_cancel_help_ohyeahcad_errtile;"
"}"
""
"ohyeahcad_button : retirement_button {"
"  key = \"ohyeahcad\";"
"  label = \"&ohyeahcad.com...\";"
"}"
""
"ok_cancel_help_ohyeahcad : column {"
"  : row {"
"    fixed_width = true;"
"    alignment = centered;"
"    ok_button;"
"    : spacer {"
"      width = 2;"
"    }"
"    cancel_button;"
"    : spacer {"
"      width = 2;"
"    }"
"    help_button;"
"    : spacer {"
"      width = 2;"
"    }"
"    ohyeahcad_button;"
"  }"
"}"
""
"ok_cancel_help_ohyeahcad_errtile : column {"
"  ok_cancel_help_ohyeahcad;"
"  errtile;"
"}"))

(setq #HELP (strcat
 "2D Sprocket Profiles\n\n"
 "This program draws 2D profiles of standard roller chain sprockets. \n"
 "You can specify the size, number of teeth, and insertion point."))

;Sprocket sizes
;
;"SIZE" "PITCH" "RLR_DIA"
(setq #SIZELIST (list
'("25" 0.25 0.13)
'("35" 0.375 0.2)
'("41" 0.5 0.306)
'("40" 0.5 0.312)
'("50" 0.625 0.4)
'("60" 0.75 0.469)
'("80" 1.0 0.625)
'("100" 1.25 0.75)
'("120" 1.5 0.875)
'("140" 1.75 1.00)
'("160" 2.0 1.125)
'("180" 2.25 1.406)
'("200" 2.5 1.562)
'("240" 3.0 1.875)))

 (defun @ARC (%A %B %C %D / #CEN)
  (setq #CEN (list (car %A) (cadr %A) (if (caddr %A) (caddr %A) 0)))
  (entmake (append (list '(0 . "ARC") '(8 . "0") (cons 10 #CEN)
   (cons 40 %B) (cons 50 %C) (cons 51 %D)))))

 (defun @BLOCK ()
  (entmake '((0 . "ENDBLK")))
  (entmake (list '(0 . "BLOCK") '(2 . "*XXX") '(70 . 1) '(10 0 0 0))))

 (defun @INSERT (%A)
  (entmake (append
    (list '(0 . "INSERT") (cons 2 (entmake '((0 . "ENDBLK")))))
     %A (list (cons 41 1.0) (cons 42 1.0) (cons 43 1.0)))))

;%A - start point
;%B - endpoint
 (defun @LINE (%A %B / #A #B)
  (setq #A (if (caddr %A) (caddr %A) 0))
  (setq #B (if (caddr %B) (caddr %B) 0))
  (entmake (append '((0 . "LINE") (8 . "0"))
   (list (list 10 (car %A) (cadr %A) #A)
         (list 11 (car %B) (cadr %B) #B)))))

 (defun @POLYARC (%PT1 %PT2 %PT3 / #1-2 #2-3 #ANG1-2 #ANG2-3 #END
  #MID #PROJ1 #PROJ2 #PT0 #START)
  (setq #ANG1-2 (angle %PT1 %PT2)
        #ANG2-3 (angle %PT2 %PT3)
        #1-2 (polar %PT1 #ANG1-2 (* 0.5 (distance %PT1 %PT2)))
        #2-3 (polar %PT2 #ANG2-3 (* 0.5 (distance %PT2 %PT3)))
        #PROJ1 (polar #1-2 (+ #ANG1-2 (* 0.5 pi)) 1.0)
        #PROJ2 (polar #2-3 (+ #ANG2-3 (* 0.5 pi)) 1.0)
        #PT0 (inters #1-2 #PROJ1 #2-3 #PROJ2 nil)
        #START (angle #PT0 %PT1)
        #MID (angle #PT0 %PT2)
        #END (angle #PT0 %PT3))
  (if (or (and (< #START #END) (or (> #MID #END) (< #MID #START)))
          (and (< #MID #START) (> #MID #END)))
   (setq #START #END #END (angle #PT0 %PT1)))
  (@ARC #PT0 (distance #PT0 %PT1) #START #END))

 (defun @POLYLINE (%PTLIST / #1 #2 #3 #FIRST #LAST 
  #PT1 #PT2 #PT3 #PTLIST)
  (setq #PTLIST %PTLIST 
        #FIRST (car #PTLIST)
        #LAST (last #PTLIST))
  (while (> (length #PTLIST) 1)
   (setq #1 (car #PTLIST)
         #2 (cadr #PTLIST)
         #3 (caddr #PTLIST)
         #PTLIST (cdr #PTLIST)
         #PT1 (list (car #1) (cadr #1))
         #PT2 (list (car #2) (cadr #2)))
   (if #3
    (setq #PT3 (list (car #3) (cadr #3))))
   (cond
    ((= (caddr #3) 1) 
     (@POLYARC #PT1 #PT2 #PT3)
     (setq #PTLIST (cdr #PTLIST)))
    (T (@LINE #PT1 #PT2)))))

 (defun @SPRKT-PTS (%PITCH %RLR_DIA %TEETH / #ANG-A #ANG-B #ANG-C
  #ANGINC #AX #CX #D #DP #INT #LAST #LIST #N #P #PD #PROFILE #PT-A #PT-B 
  #PT-BPEAK #PT-C #PT-INT #PT-PEAK #PT-X #PT-XY #PT-XX #PT-Z #PT0 #PTLIST 
  #RAD #RAD-P #TMP #X) 
  (setq #ANG 0
        #ANGINC (/ (* 2.0 pi) %TEETH)
        #PT0 '(0 0)
        #N %TEETH
        #PD (/ %PITCH (sin (/ pi #N)))
        #RAD-P (* 0.5 #PD)
        #PT-A (list #RAD-P 0)
        #D %RLR_DIA
        #DP (+ (* 1.005 %RLR_DIA) 0.003)
        #ANG-A (+ (* pi (/ 35.0 180.0)) (/ pi 3.0 #N))
        #ANG-B (- (* pi 0.1) (* pi (/ 56.0 180.0 #N)))
        #ANG-C (/ pi #N)
        #AX (* 0.5 #DP)
        #PT-X (polar #PT-A (- (* 1.5 pi) #ANG-A) #AX)
        #PT-XX (polar #PT-A pi #AX)
        #PT-C (polar #PT-A (- (* 0.5 pi) #ANG-A) (* 0.8 #D))
        #CX (distance #PT-C #PT-X)
        #PT-Y (polar #PT-C (- (* 1.5 pi) #ANG-A (- #ANG-B)) #CX)
        #PT-XY (polar #PT-C (- (* 1.5 pi) #ANG-A (* -0.5 #ANG-B)) #CX)
        #PT-Z (polar #PT-Y (- #ANG-B #ANG-A) 1)
        #PT-B (polar #PT-A (- (* 1.5 pi) #ANG-C) (* 1.24 #D)) 
        #PT-Z (inters #PT-Y #PT-Z #PT-B
         (polar #PT-B (- (* 1.5 pi) #ANG-A (- #ANG-B)) 1) nil)
        #RAD-F (distance #PT-B #PT-Z)
        #PT-INT (inters #PT0 (polar #PT0 (- #ANG-C) 1)
        #PT-B (polar #PT-B (- (* 0.5 pi) #ANG-C) 1) nil)
        #INT (distance #PT-B #PT-INT)
        #INT (sqrt (- (expt #RAD-F 2.0) (expt #INT 2.0))) 
        #PT-PEAK (polar #PT-INT (- #ANG-C) #INT)
        #RAD (distance #PT0 #PT-PEAK)
        #PT-BPEAK (polar #PT-B
         (* 0.5 (+ (angle #PT-B #PT-PEAK) (angle #PT-B #PT-Z))) #RAD-F)
        #PTLIST (list #PT-PEAK #PT-BPEAK (list (car #PT-Z) (cadr #PT-Z) 1)
         #PT-Y #PT-XY (list (car #PT-X) (cadr #PT-X) 1) #PT-XX
         (list (car #PT-X) (- (cadr #PT-X)) 1)
         (list (car #PT-XY) (- (cadr #PT-XY)))
         (list (car #PT-Y) (- (cadr #PT-Y)) 1)
         (list (car #PT-Z) (- (cadr #PT-Z)))
         (list (car #PT-BPEAK) (- (cadr #PT-BPEAK)))
         (list (car #PT-PEAK) (- (cadr #PT-PEAK)) 1)))
   (repeat %TEETH
    (foreach #X (cdr #PTLIST)
     (setq #P (list (car #X) (cadr #X))
           #TMP (polar #PT0 (+ (angle #PT0 #P) #ANG) (distance #PT0 #P)))
     (if (caddr #X) (setq #TMP (list (car #TMP) (cadr #TMP) 1)))
     (setq #LIST (cons #TMP #LIST)))
    (setq #ANG (+ #ANG #ANGINC)))
   (setq #LIST (append #LIST (list (car #LIST))))
   (reverse #LIST))

 (defun @SPROCKET-DRAW (/ #PT0 #PTS)
  (initget 1)
  (setq #PT0 (getpoint "\nInsert point: ")
        #PTS (@SPRKT-PTS (cadr #SIZE) (caddr #SIZE) #TEETH))
  (@BLOCK)
  (@POLYLINE #PTS)
  (@INSERT (list (cons 10 #PT0))))

 (defun @TEETH (%A / #TMP)
  (if (> (setq #TMP (abs (atoi %A))) 8) (setq #TEETH #TMP))
  (set_tile "teeth" (itoa #TEETH)))

 (setvar "cmdecho" 0)
 (setq #DCL-FILE (vl-filename-mktemp "2Dsprkt.dcl")
       #FILE (open #DCL-FILE "w"))
 (foreach #X #DCL-LIST (write-line #X #FILE))
 (close #FILE)
 (if (< (setq #DCL-ID (load_dialog #DCL-FILE)) 0)
  (progn
   (alert "\nCan't load DCL file.")
   (quit))
  (vl-file-delete #DCL-FILE))
 (if (not (new_dialog "sprocket2d" #DCL-ID)) (quit))
 (start_list "size")
 (foreach #X #SIZELIST
  (add_list (car #X)))
 (end_list)
 (setq #SIZE (assoc "35" #SIZELIST))
 (set_tile "size" (itoa (vl-position #SIZE #SIZELIST)))
 (@TEETH "12")
 (action_tile "accept" "(done_dialog 1)")
 (action_tile "ohyeahcad" "(done_dialog 2)")
 (action_tile "help" "(alert #HELP)")
 (action_tile "size" "(setq #SIZE (nth (atoi $value) #SIZELIST))")
 (action_tile "teeth" "(@TEETH $value)")
 (action_tile "teeth_minus" "(@TEETH (itoa (1- #TEETH)))")
 (action_tile "teeth_plus" "(@TEETH (itoa (1+ #TEETH)))")
 (setq #GO (start_dialog))
 (cond
  ((= #GO 1) (@SPROCKET-DRAW))
  ((= #GO 2) (command "_.browser" "www.ohyeahcad.com")))
 (princ))


;;; Uncomment for the language needed.
;(princ "\n\n2DSPRKT ͂ĊJn")  ; For Japanese.
(princ "\n\nType 2DSPRKT to start.") ; For English.

(princ)
